RelocatablePointer

Struct RelocatablePointer 

pub struct RelocatablePointer<T> { /* private fields */ }
A RelocatablePointer stores only the distance from its memory starting position to the memory location it is pointing to. When the RelocatablePointer is now shared between processes its virtual memory starting position changes but the distance to the object it is pointing to is the same.

Important:

  1. Every construct which relies on a RelocatablePointer must be declared with [repr(C)]. Otherwise different compilation units may have a different structural layout of the data type which is shared between processes which leads to undefined behavior.
  2. The construct which is using the RelocatablePointer and the pointee must be stored in the same shared memory object. Pointing to a different shared memory segment most likely leads to crashes since it can be mapped in a different order, at a different position and the distance to the memory destination is off.

Implementations

Source

impl<T> RelocatablePointer<T>

pub fn new(distance: isize) -> Self

Creates a new RelocatablePointer. The distance is the relative distance to the memory destination starting from the memory location of this RelocatablePointer.

pub unsafe fn new_uninit() -> Self

Creates a new uninitialized RelocatablePointer.

Safety
pub unsafe fn init(&self, ptr: NonNull<[u8]>)

Initializes the RelocatablePointer by setting the distance to the memory destination by providing an absolut pointer to it. The distance can be calculated from the RelocatablePointer memory location and the absolut position of the destination. Important: The pointer must point into the same shared memory object.

Safety